<?php
///////////////////////////////////////////////////////////////////////////////
//                                                                           //
// Copyright (C) 2009  Phorum Development Team                               //
// http://www.phorum.org                                                     //
//                                                                           //
// This program is free software. You can redistribute it and/or modify      //
// it under the terms of either the current Phorum License (viewable at      //
// phorum.org) or the Phorum License that was distributed with this file     //
//                                                                           //
// This program is distributed in the hope that it will be useful,           //
// but WITHOUT ANY WARRANTY, without even the implied warranty of            //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      //
//                                                                           //
// You should have received a copy of the Phorum License                     //
// along with this program.                                                  //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////

if(!defined("PHORUM")) return;

// The path to the HTML Purifier stand alone distribution.
define('HTMLPURIFIER_PATH',dirname(__FILE__).'/htmlpurifier-4.0.0-standalone');

// Load HTMLPurifier.
ini_set(
    'include_path',
    ini_get('include_path') . PATH_SEPARATOR .
    HTMLPURIFIER_PATH.'/standalone'
);
require HTMLPURIFIER_PATH.'/HTMLPurifier.standalone.php';

// HTML Phorum Mod

function phorum_mod_html_posting_custom_action($message)
{
    global $PHORUM;
    $mod_html = isset($PHORUM['mod_html']) ? $PHORUM['mod_html'] : array();
    
    // For backwards compatibility, the default state is everyone is enabled to enter HTML.
    // We detect this state by checking for one of the settings, if it doesn't exist then the
    // administrator hasn't modified the module settings, so we remain in the default:
    if (!isset($mod_html['anyone_enabled']))
        return $message;

    if ($mod_html['anyone_enabled']) {
        // If true, then everyone is enabled to enter HTML
        $message["meta"]["mod_html"]["authorize"] = true;
        return $message;
    }
    else
        $message["meta"]["mod_html"]["authorize"] = false;

    // Additional conditions depend on a user
    $forum_id = $message['forum_id'];
    $uid = isset($message["user_id"]) ? $message["user_id"] : 0 ;
    if (!$uid)
        return $message;

    $user = phorum_db_user_get($uid, true);
    if ($user['admin']) {
        $message["meta"]["mod_html"]["authorize"] = true;
        return $message;
    }
    $authorize = $mod_html['users_enabled'] || $mod_html['groups_enabled'];  // $authorize flag starts out true for logged in users.
    // In order to remain true, it will have to pass
    // any and all checks configured by the module's admin settings.  Any checks that fail
    // stop any further checks.

    if ($authorize && $mod_html['groups_enabled']) {
        $authorize = false;     // set the flag false and this check has to change it back to true
        if (!empty($user['groups'])) {
            // Is user an approved member of a selected group?
            foreach($mod_html['groups'] as $group) {
                $group_id = $group["group_id"];
                $authorize = (isset($user['groups'][$group_id]));
                if ($authorize && $group['active_forum']) {
                    // The group must be attached to the active forum
                    $group_info = phorum_db_get_groups($group_id);
                    $authorize = isset($group_info[$group_id]["permissions"][$forum_id]);
                }
    
                if (!$authorize) break;
            }
        }
    }

    if ($authorize && $mod_html['users_enabled']) {

        // Does user match any of the following criteria checks?

        // Is the user a moderator in the active forum...
        if ($mod_html['user_is_moderator']) {
            $authorize = false;     // set the flag false and this check has to change it back to true

//            $authorize = phorum_api_user_check_access(PHORUM_USER_ALLOW_MODERATE_MESSAGES, $forum_id, $user);

            // ...based on group membership?
            if (isset($user["group_permissions"][$forum_id])) {
                $perms = $user["group_permissions"][$forum_id];
                $authorize = ($perms & PHORUM_USER_ALLOW_MODERATE_MESSAGES) > 0;
            }


            // ...based on personal permissions?
            if ($authorize && isset($user["forum_permissions"][$forum_id])) {
                $perms = $user["forum_permissions"][$forum_id];
                $authorize = ($perms & PHORUM_USER_ALLOW_MODERATE_MESSAGES) > 0;
            }
        }

        // Is the user post count is equal to or more than NNN
        if ($authorize && is_numeric($mod_html['users_min_post_count'])) {
            $authorize = false;     // set the flag false and this check has to change it back to true
            $authorize = (intval($user['posts']) >= $mod_html['users_min_post_count']);
        }

        // Is the user registration date is equal to or more than NNN
        if ($authorize && is_numeric($mod_html['users_min_registration_age'])) {
            $authorize = false;     // set the flag false and this check has to change it back to true
            $date_added = isset($user['raw_date_added'])
                        ? $user['raw_date_added'] : $user['date_added'];
            $authorize = (((time() - $date_added) / 86400) >= $mod_html['users_min_registration_age']);
        }

        // Integration with User Tagging module:
        // Is the user tagged by any selected rules
        if ($authorize && isset($PHORUM['mod_user_tagging']) && $PHORUM["mods"]["user_tagging"] && !empty($PHORUM['mod_user_tagging']['rules']) && isset($mod_html['user_tagging_rules']))
        {
            $authorize = false;     // set the flag false and this check has to change it back to true
            require_once('./mods/user_tagging/api.php');
            foreach ($mod_html['user_tagging_rules'] as $rule_id => $rule_status) {
                if ($rule_status) {
                    if (!isset($PHORUM['mod_user_tagging']['rules'][$rule_id]))
                        continue;

                    $user_filters = true;
                    $value = user_tagging_process_rule($PHORUM['mod_user_tagging']['rules'][$rule_id], $user);
                    $authorize = ($value !== NULL);
                }
                if (!$authorize)
                    break;
            }
        }

    }

    $message["meta"]["mod_html"]["authorize"] = $authorize;

    return $message;
}

function phorum_mod_html_format($data)
{
    global $PHORUM;
    static $purifier;
    static $config;

    // Setup the HTML Purifier object.
    if (!$purifier)
    {
        $cache = $PHORUM['cache'] . '/html_purifier';
        if (!file_exists($cache) && !mkdir($cache)) trigger_error(
            "The HTML module is unable to create the HTML Purifier " .
            "cache directory \"$cache\". Fix the cause of this problem " .
            "or disable the HTML module in the Phorum admin interface.",
            E_USER_ERROR
        );

        // Determine the doctype to use.
        $doctype = isset($PHORUM['mod_html']['doctype'])
                 ? $PHORUM['mod_html']['doctype']
                 : 'XHTML 1.0 Transitional';

        // Bootstrap the HTML Purifier.
        $config = HTMLPurifier_Config::createDefault();
        $config->set('Core.Encoding', $PHORUM['DATA']['CHARSET']);
        $config->set('HTML.Doctype', $doctype);
        $config->set('Cache.SerializerPath', $cache);
        $purifier = new HTMLPurifier($config);
    }

    $PHORUM = $GLOBALS["PHORUM"];

    foreach($data as $message_id => $message)
    {
        if(isset($message["body"]))
        {
            // Look for the "authorize" token stored in the message's meta data.
            // If it exists, then it must be true in order for HTML tags to be restored.
            // If it doesn't exist, then we are in a backwards compatibility mode in which *all* HTML
            // input is allowed when the module is enabled.
            if (isset($message["meta"]["mod_html"]["authorize"]))
            {
                if (!$message["meta"]["mod_html"]["authorize"])
                    continue;       // HTML input not explicitly not authorized
            }
            //  the else is an implicit allowance of HTML input

            $body = $message["body"];

            // pull out the phorum breaks
            $body = str_replace("<phorum break>", "", $body);

            // Protect against poisoned null byte XSS attacks
            // (MSIE does not protect itself against these, so we have
            // to take care of that).
            str_replace("\0", "", $body);

            // restore tags where Phorum has killed them
            $body = preg_replace("!&lt;(\/*[a-z].*?)&gt;!si", "<$1>", $body);

            // restore escaped & and "
            $body = str_replace("&amp;", "&", $body);
            $body = str_replace("&quot;", '"', $body);

            // run the message through HTML Purifier for stripping out
            // possible XSS risks.
            $body = $purifier->purify($body);

            // put the phorum breaks back
            $body = str_replace("\n", "<phorum break>\n", $body);

            // strip any <phorum break> tags that got inside certain
            // blocks like tables (to prevent <table><br/><tr> like
            // code) and pre/xmp (newlines are shown, even without
            // <br/> tags).
            $block_tags="table|pre|xmp";

            preg_match_all("!(<($block_tags).*?>).+?(</($block_tags).*?>)!ims", $body, $matches);

            foreach($matches[0] as $block){
                $newblock=str_replace("<phorum break>", "", $block);
                $body=str_replace($block, $newblock, $body);
            }

            $data[$message_id]["body"] = $body;
        }
    }

    return $data;
}

?>
